<?php
/** 
*
* @package install
* @version $Id:
* @copyright (c) 2006 phpBB Group 
* @copyright (c) Dicky 2008
* @copyright (c) prototech 2013
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/

/**
* Helper functions for vBulletin 4.x.x to phpBB 3.0.x conversion
*/

/**
* Counterpart to is_empty()
*/
function not_is_empty($mixed)
{
	return (empty($mixed)) ? false : true;
}

/**
* Function for recoding text with the default language
*
* @param string $text text to recode to utf8
*/
function vb_set_encoding($text)
{
	global $src_db, $same_db, $convert;
	static $encoding;

	if ($encoding == null)
	{
		if ($convert->mysql_convert && $same_db)
		{
			$src_db->sql_query("SET NAMES 'binary'");
		}

		$sql = 'SELECT l.charset
			FROM ' . $convert->src_table_prefix . 'language l, ' .
				$convert->src_table_prefix . 'setting s
			WHERE s.varname = "languageid"
				AND l.languageid = s.value';
		$result = $src_db->sql_query($sql);
		$encoding = $src_db->sql_fetchfield('charset');
		$src_db->sql_freeresult($result);

		if ($convert->mysql_convert && $same_db)
		{
			$src_db->sql_query("SET NAMES 'utf8'");
		}
	}

	return utf8_recode($text, $encoding);
}

/**
* Return correct user id value
* Everyone's id will be one higher to allow the guest/anonymous user to have a positive id as well
*/
function phpbb_user_id($user_id)
{
	global $config;

	// Increment user id if the old forum is having a user with the id 1
	if (!isset($config['increment_user_id']))
	{
		global $src_db, $same_db, $convert;

		if ($convert->mysql_convert && $same_db)
		{
			$src_db->sql_query("SET NAMES 'binary'");
		}

		// Now let us set a temporary config variable for user id incrementing
		$sql = "SELECT userid
			FROM {$convert->src_table_prefix}user
			WHERE userid = 1";
		$result = $src_db->sql_query($sql);
		$id = (int) $src_db->sql_fetchfield('userid');
		$src_db->sql_freeresult($result);

		// Try to get the maximum user id possible...
		$sql = "SELECT MAX(userid) AS max_user_id
			FROM {$convert->src_table_prefix}user";
		$result = $src_db->sql_query($sql);
		$max_id = (int) $src_db->sql_fetchfield('max_user_id');
		$src_db->sql_freeresult($result);

		if ($convert->mysql_convert && $same_db)
		{
			$src_db->sql_query("SET NAMES 'utf8'");
		}

		// If there is a user id 1, we need to increment user ids. :/
		if ($id === 1)
		{
			set_config('increment_user_id', ($max_id + 1), true);
			$config['increment_user_id'] = $max_id + 1;
		}
		else
		{
			set_config('increment_user_id', 0, true);
			$config['increment_user_id'] = 0;
		}
	}

	// If the old user id is -1 in 2.0.x it is the anonymous user...
	if ($user_id == -1)
	{
		return ANONYMOUS;
	}
	if ($user_id == 0)
	{
		return ANONYMOUS;
	}

	if (!empty($config['increment_user_id']) && $user_id == 1)
	{
		return (int) $config['increment_user_id'];
	}

	return (int) $user_id;
}

/**
* Determine the file extension using the file name.
*/
function vb_file_ext($filename)
{
	return strtolower(substr(strrchr($filename,'.'),1));
}

/**
* Set forum flags - only prune old polls by default
*/
function phpbb_forum_flags()
{
	// Set forum flags
	$forum_flags = 0;

	// FORUM_FLAG_LINK_TRACK
	$forum_flags += 0;

	// FORUM_FLAG_PRUNE_POLL
	$forum_flags += FORUM_FLAG_PRUNE_POLL;

	// FORUM_FLAG_PRUNE_ANNOUNCE
	$forum_flags += 0;

	// FORUM_FLAG_PRUNE_STICKY
	$forum_flags += 0;

	// FORUM_FLAG_ACTIVE_TOPICS
	$forum_flags += 0;

	// FORUM_FLAG_POST_REVIEW
	$forum_flags += FORUM_FLAG_POST_REVIEW;

	return $forum_flags;
}

/**
* Calculate the left right id's for forums. This is a recursive function.
*/
function vb_left_right_ids($groups, $parent_id, &$forums, &$node)
{
	foreach ($groups[$parent_id] as $forum_id)
	{
		$forums[$forum_id]['left_id'] = $node++;

		if (!empty($groups[$forum_id]))
		{
			vb_left_right_ids($groups, $forum_id, $forums, $node);
		}

		$forums[$forum_id]['right_id'] = $node++;
	}
}

/**
* Insert/Convert forums
*/
function vb_insert_forums()
{
	global $db, $src_db, $same_db, $convert;

	$db->sql_query($convert->truncate_statement . FORUMS_TABLE);

	if ($convert->mysql_convert && $same_db)
	{
		$src_db->sql_query("SET NAMES 'binary'");
	}

	$sql = 'SELECT forumid, title, description, options, displayorder, parentid, password, link
		FROM ' . $convert->src_table_prefix . 'forum
		ORDER BY parentid, displayorder ASC';
	$result = $src_db->sql_query($sql);
	$forums = $forum_groups = array();

	while ($row = $src_db->sql_fetchrow($result))
	{
		$row['parentid'] = ($row['parentid'] == -1) ? 0 : $row['parentid'];

		if (!empty($row['link']))
		{
			$row['forum_type'] = FORUM_LINK;
		}
		else if ($row['options'] & 4)
		{
			$row['forum_type'] = FORUM_POST;
		}
		else
		{
			$row['forum_type'] = FORUM_CAT;
		}

		$forums[$row['forumid']] = $row;
		$forum_groups[$row['parentid']][] = $row['forumid'];
	}
	$src_db->sql_freeresult($result);

	if ($convert->mysql_convert && $same_db)
	{
		$src_db->sql_query("SET NAMES 'utf8'");
	}

	switch ($db->sql_layer)
	{
		case 'mssql':
		case 'mssql_odbc':
			$db->sql_query('SET IDENTITY_INSERT ' . FORUMS_TABLE . ' ON');
		break;
	}

	// Calculate the left and right id's
	$node = 1;
	vb_left_right_ids($forum_groups, 0, $forums, $node);

	foreach ($forums as $forum_id => $row)
	{
		$row['description'] = preg_replace('#<(br|br/|br /|br\s/)>#i', "\n", $row['description']);

		$sql_ary = array(
			'forum_id'			=> (int) $forum_id,
			'forum_name'		=> utf8_htmlspecialchars(vb_set_encoding($row['title'])),
			'parent_id'			=> (int) $row['parentid'],
			'forum_parents'		=> '',
			'forum_desc'		=> htmlspecialchars(vb_set_encoding(htmlspecialchars_decode(html_entity_decode($row['description']), ENT_QUOTES)), ENT_COMPAT, 'UTF-8'),
			'forum_type'		=> $row['forum_type'],
			'forum_status'		=> ($row['options'] & 2) ? ITEM_UNLOCKED : ITEM_LOCKED,
			'forum_link'		=> $row['link'],
			'forum_password'	=> $row['password'],
			'forum_flags'		=> phpbb_forum_flags(),
			'left_id'			=> $row['left_id'],
			'right_id'			=> $row['right_id'],
			'enable_icons'		=> 1,

			// Default values
			'forum_desc_bitfield'		=> '',
			'forum_desc_options'		=> 7,
			'forum_desc_uid'			=> '',
			'forum_style'				=> 0,
			'forum_image'				=> '',
			'forum_rules'				=> '',
			'forum_rules_link'			=> '',
			'forum_rules_bitfield'		=> '',
			'forum_rules_options'		=> 7,
			'forum_rules_uid'			=> '',
			'forum_topics_per_page'		=> 0,
			'forum_posts'				=> 0,
			'forum_topics'				=> 0,
			'forum_topics_real'			=> 0,
			'forum_last_post_id'		=> 0,
			'forum_last_poster_id'		=> 0,
			'forum_last_post_subject'	=> '',
			'forum_last_post_time'		=> 0,
			'forum_last_poster_name'	=> '',
			'forum_last_poster_colour'	=> '',
			'display_on_index'			=> 1,
			'enable_indexing'			=> 1,
		);

		$sql = 'INSERT INTO ' . FORUMS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
	}

	switch ($db->sql_layer)
	{
		case 'mssql':
		case 'mssql_odbc':
			$db->sql_query('SET IDENTITY_INSERT ' . FORUMS_TABLE . ' OFF');
		break;
	}
}

/**
* Import topic/post icons
*/
function vb_import_icon($source)
{
	global $convert;

	$dim = get_image_dim($source);
	$source = $convert->convertor_data['forum_path'] . '/' . $source;
	$result = _import_check('icons_path', $source, false);

	if (!$result['copied'])
	{
		$result['target'] = utf8_basename($result['target']);
	}
	if (empty($dim))
	{
		$dim = array(0, 0);
	}

	$convert->row['icon_height'] = $dim[0];
	$convert->row['icon_width'] = $dim[1];

	return $result['target'];
}

/**
* Fetch the topic/post icon height.
*/
function vb_icon_height()
{
	global $convert;

	return $convert->row['icon_height'];
}

/**
* Fetch the topic/post icon width.
*/
function vb_icon_width()
{
	global $convert;

	return $convert->row['icon_width'];
}

/**
* Determine how many poll options can be voted for.
*/
function vb_poll_options($multiple)
{
	global $convert_row;

	return (!$multiple) ? 1 : (int) $convert_row['numberoptions'];
}

/**
* Extract poll options and votes from strings and insert them into the poll options database.
*/
function vb_insert_poll_options()
{
	global $db, $convert_row;

	$options = explode('|||', $convert_row['options']);
	$votes = explode('|||', $convert_row['votes']);
	$sql_ary = array();
	$i = 1;

	foreach ($options as $index => $option)
	{
		$sql_ary[] = array(
			'poll_option_id'	=> $i,
			'topic_id'			=> (int) $convert_row['threadid'],
			'poll_option_text'	=> vb_set_encoding($option),
			'poll_option_total'	=> (int) $votes[$index]
		);
		$i++;
	}

	if (!empty($sql_ary))
	{
		$db->sql_multi_insert(POLL_OPTIONS_TABLE, $sql_ary);
	}
}

/**
* Determine if the image has a thumbnail.
*/
function vb_attach_has_thumbnail()
{
	global $convert_row;

	if ($convert_row['height'] > VB_THUMB_LIMIT || $convert_row['width'] > VB_THUMB_LIMIT)
	{
		return 1;
	}
	return 0;
}

/**
* Import attachment file
*/
function vb_import_attachment($user_id)
{
	global $convert, $convert_row;

	$user_id = (int) $user_id;
	$target = phpbb_user_id($user_id) . '_' . md5(unique_id());
	$storage_method = get_config_value('attachfile');
	$thumbnail = vb_attach_has_thumbnail();

	// Attachments are stored in the database if $storage_method is 0.
	if ($storage_method < 1)
	{
		vb_write_file('upload_path', $convert_row['filedata'], $target);

		if ($thumbnail)
		{
			vb_write_file('upload_path', $convert_row['thumbnail'], 'thumb_' . $target);
		}
		return $target;
	}
	else
	{
		$dir = ($storage_method == 2) ? $user_id . '/' : '';
		$source = $dir . $convert_row['attachmentid'] . '.attach';
		// The attachment path is absolute, so we'll temporarily set this setting to true so that that import_attachment() works correctly.
		$convert->convertor['source_path_absolute'] = true;

		if ($thumbnail)
		{
			$thumbnail = $dir . $convert_row['attachmentid'] . '.thumb';
			import_attachment($thumbnail, 'thumb_' . $target);
		}

		import_attachment($source, $target);
		$convert->convertor['source_path_absolute'] = false;

		return $target;
	}

	return '';
}

/**
* Write database file data to the filesystem
*/
function vb_write_file($config_var, $file_data, $target)
{
	global $config, $phpbb_root_path, $convert;

	$write_dir = $phpbb_root_path . $config[$config_var] . '/';

	if (!($fp = fopen($write_dir . $target, 'w')) )
	{
		$lang['INST_ERR_FATAL'] = $user->lang['CONV_ERR_FATAL'];
		$convert->p_master->error('<span style="color:red">Unable to write to ' . htmlspecialchars($write_dir) . '</span>', __LINE__, __FILE__);
	}
	else
	{
		fwrite($fp, $file_data);
		fclose($fp);
	}
}

/**
* Convert the group name, making sure to avoid conflicts with 3.0 special groups
*/
function phpbb_convert_group_name($group_name)
{
	$default_groups = array(
		'GUESTS',
		'REGISTERED',
		'REGISTERED_COPPA',
		'GLOBAL_MODERATORS',
		'ADMINISTRATORS',
		'BOTS',
	);

	if (in_array(strtoupper($group_name), $default_groups))
	{
		return 'vB - ' . $group_name;
	}

	return utf8_htmlspecialchars(vb_set_encoding($group_name));
}

/**
* Convert the group type constants
*/
function phpbb_convert_group_type($group_public)
{
	global $convert_row;

	// The first 8 groups in vB are special groups. These aren't visible to users.
	if ($convert_row['usergroupid'] <= 8)
	{
		return GROUP_HIDDEN;
	}

	return ($group_public) ? GROUP_OPEN : GROUP_CLOSED;
}

function vb_replace_size($matches)
{
	return '[size=' . min(200, ceil(100.0 * (((double) $matches[1])/12.0))) . ']';
}

/**
* Wrap smiley codes with space so phpBB can parse them correctly.
*/
function vb_reformat_smilies(&$message)
{
	global $convert, $src_db, $same_db;
	static $smilies;

	if ($smilies === null)
	{
		if ($convert->mysql_convert && $same_db)
		{
			$src_db->sql_query("SET NAMES 'binary'");
		}

		$sql = 'SELECT smilietext
			FROM ' . $convert->src_table_prefix . 'smilie';
		$result = $src_db->sql_query($sql);
		$smilies = array();

		while ($row = $src_db->sql_fetchrow($result))
		{
			$smilies[] = $row['smilietext'];
		}
		$src_db->sql_freeresult($result);

		if ($convert->mysql_convert && $same_db)
		{
			$src_db->sql_query("SET NAMES 'utf8'");
		}
	}

	foreach ($smilies as $smiley)
	{
		if (strpos($message, $smiley) !== false)
		{
			$search = '#([^\s])?(' . preg_quote($smiley, '#') . ')([^\s])?#';
			$message = preg_replace_callback($search, 'vb_smiley_space', $message);
		}
	}
}

/**
* Wrap smiley code with space.
*/
function vb_smiley_space($match)
{
	$replace = (!empty($match[1])) ? $match[1] . ' ' : '';
	$replace .= $match[2];
	$replace .= (!empty($match[3])) ? ' ' . $match[3] : '';

	return $replace;
}

/**
* Reformat inline attachment bbcode to the proper phpBB format.
*/
function vb_reformat_inline_attach(&$message, $post_id)
{
	global $src_db, $convert, $same_db;

	// Do a simple check for the presence of opening and closing tags before continuing.
	if (strpos($message, '[ATTACH=CONFIG]') !== false && strpos($message, '[/ATTACH]') !== false)
	{
		if ($convert->mysql_convert && $same_db)
		{
			$src_db->sql_query("SET NAMES 'binary'");
		}
		// We need to grab some info from the database :-/
		$sql = 'SELECT attachmentid, filename
			FROM ' . $convert->src_table_prefix . 'attachment
			WHERE contenttypeid = 1
				AND contentid = ' . (int) $post_id . '
			ORDER BY attachmentid DESC';
		$result = $src_db->sql_query($sql);
		$i = 1;

		while ($row = $src_db->sql_fetchrow($result))
		{
			$find = '[ATTACH=CONFIG]' . $row['attachmentid'] . '[/ATTACH]';
			$replace = '[attachment=' . $i . ']' . $row['filename'] . '[/attachment]';

			$message = str_replace($find, $replace, $message); 
		}
		$src_db->sql_freeresult($result);

		if ($convert->mysql_convert && $same_db)
		{
			$src_db->sql_query("SET NAMES 'utf8'");
		}
	}
}

/**
* Reparse the message stripping out the bbcode_uid values and adding new ones and setting the bitfield
* @todo What do we want to do about HTML in messages - currently it gets converted to the entities, but there may be some objections to this
*/
function vb_prepare_message($message)
{
	global $convert, $user, $convert_row, $message_parser;

	if (!$message)
	{
		$convert->row['mp_bbcode_bitfield'] = $convert_row['mp_bbcode_bitfield'] = 0;
		return '';
	}
	
	$message = preg_replace('#<(br|br/|br /|br\s/)>#i', "\n", $message);

	if (!empty($convert_row['allowsmilie']))
	{
		vb_reformat_smilies($message);
	}

	// Convert inline attachment bbcode
	if (!empty($convert_row['attach']))
	{
		vb_reformat_inline_attach($message, $convert_row['postid']);
	}

	$message = preg_replace('#\[glow=(.*?):(.*?)\]#i', "[glow=\\1]", $message);
	$message = preg_replace('#\[shadow=(.*?):(.*?)\]#i', "[shadow=\\1]", $message);
	$message = preg_replace('#\[\/glow:(.*?)\]#i', "[/glow]", $message);
	$message = preg_replace('#\[\/shadow:(.*?)\]#i', "[/shadow]", $message);
	$message = preg_replace( '/\[url=\"(.+?)\"]/si', "[url=\\1]", $message );
	$message = preg_replace( '/\[url=\'(.+?)\']/si', "[url=\\1]", $message );

	$bbcode_replacements = array(
		'[center]'	=> '[align=center]',
		'[/center]'	=> '[/align]',
		'<center>'	=> '[align=center]',
		'</center>' => '[/align]',
		'[left]'	=> '[align=left]',
		'[/left]'	=> '[/align]',
		'[right]'	=> '[align=right]',
		'[/right]'	=> '[/align]',
		'[li]'		=> '[*]',
		'[FONT='	=> '[font=',
		'[/FONT]'	=> '[/font]',
		'[COLOR='	=> '[color=',
		'[/COLOR]'	=> '[/color]',
		'[QUOTE'	=> '[quote',
		'[PHP]'		=> '[code=php]',
		'[/PHP]'	=> '[/code]',
		'[/SIZE]'	=> '[/size]',
	);

	$message = str_replace(array_keys($bbcode_replacements), $bbcode_replacements, $message);

	//	From vb3 to phpBB2 convertor
	$search = "/\[size=([0-9]+)\]/ie";
	$replace = "'[size='.( (\\1 == 1) ? 9 : ( (\\1 == 2) ? 10 : 4*\\1 )) .']'";
		
	if (stripos($message, '[size=') !== false)
	{
		$message = preg_replace($search, $replace, $message);
		$message = preg_replace_callback('/\[size=(\d*)\]/i', 'vb_replace_size', $message);
	}

	if (strpos($message, '[quote=') !== false)
	{
		$message = preg_replace('/\[quote="(.*?)"\]/s', '[quote=&quot;\1&quot;]', $message);
		$message = preg_replace('/\[quote=(.*?);(.*?)\]/s', '[quote=\1]', $message);
		$message = preg_replace('/\[quote=(.*?)\]/s', '[quote=&quot;\1&quot;]', $message);
	}

	// Already the new user id ;)
	$user_id = $convert->row['poster_id'];

	$message = str_replace('<', '&lt;', $message);
	$message = str_replace('>', '&gt;', $message);

	// make the post UTF-8
	$message = vb_set_encoding($message);

	$message_parser->warn_msg = array(); // Reset the errors from the previous message
	$message_parser->bbcode_uid = make_uid($convert->row['post_time']);
	$message_parser->message = $message;
	unset($message);

	// Make sure options are set.
	$enable_bbcode = (!isset($convert->row['enable_bbcode'])) ? true : $convert->row['enable_bbcode'];
	$enable_smilies = (!isset($convert->row['allowsmilie'])) ? true : $convert->row['allowsmilie'];
	$enable_magic_url = (!isset($convert->row['enable_magic_url'])) ? true : $convert->row['enable_magic_url'];

	// parse($allow_bbcode, $allow_magic_url, $allow_smilies, $allow_img_bbcode = true, $allow_flash_bbcode = true, $allow_quote_bbcode = true, $allow_url_bbcode = true, $update_this_message = true, $mode = 'post')
	$message_parser->parse($enable_bbcode, $enable_magic_url, $enable_smilies);
	
	if (sizeof($message_parser->warn_msg))
	{
		$msg_id = isset($convert->row['post_id']) ? $convert->row['post_id'] : $convert->row['privmsgs_id'];
		$convert->p_master->error('<span style="color:red">' . $user->lang['POST_ID'] . ': ' . $msg_id . ' ' . $user->lang['CONV_ERROR_MESSAGE_PARSER'] . ': <br /><br />' . implode('<br />', $message_parser->warn_msg), __LINE__, __FILE__, true);
	}

	$convert->row['mp_bbcode_bitfield'] = $convert_row['mp_bbcode_bitfield'] = $message_parser->bbcode_bitfield;

	$message = $message_parser->message;
	unset($message_parser->message);

	return $message;
}

/**
* Return the bitfield calculated by the previous function
*/
function get_bbcode_bitfield()
{
	global $convert_row;

	return $convert_row['mp_bbcode_bitfield'];
}

/**
* Determine the last user to edit a post
* In practice we only tracked edits by the original poster in 2.0.x so this will only be set if they had edited their own post
*/
function phpbb_post_edit_user()
{
	global $convert_row, $config;

	if (isset($convert_row['post_edit_count']))
	{
		return phpbb_user_id($convert_row['poster_id']);
	}

	return 0;
}

/**
* Calculate the correct to_address field for private messages
*/
function vb_privmsgs_to_users($user_array)
{
	$users = unserialize($user_array);
	$to = '';

	foreach ($users['cc'] as $user_id => $username)
	{
		$to .= ((!empty($to)) ? ', ' : '') . 'u_' . phpbb_user_id($user_id);
	}

	return $to;
}

/**
* Calculate whether a private message was unread using the bitfield
*/
function vb_unread_pm($messageread)
{
	return ($messageread == 0) ? 1 : 0;
}

/**
* 没回复就算新的？
*/
function vb_new_pm($messageread)
{
	return ($messageread == 2) ? 0 : 1;
}
function vb_replied_pm($messageread)
{
	return ($messageread == 2) ? 1 : 0;
}

/**
* Convert birthday to phpBB Format
*/
function vb_get_birthday($birthday = '')
{
	$birthday = (string) $birthday;

	// stored as month, day, year
	if (!$birthday)
	{
		return ' 0- 0-   0';
	}

	// Expected format from vB3 is MM-DD-YYYY
	$birthday_parts = explode('-',$birthday);

	$month = $birthday_parts[0];
	$day = $birthday_parts[1];
	$year =  $birthday_parts[2];

	return sprintf('%2d-%2d-%4d', $day, $month, $year);
}

/**
* Set primary group.
*/
function vb_set_primary_group($group_id)
{
	if ($group_id == 6)
	{
		return get_group_id('administrators');
	}
	else if ($group_id == 3)
	{
		// Group 3 is the Users Awaiting Email Confirmation group. These users not activated in phpBB.
		return 0;
	}
	else
	{
		return get_group_id('registered');
	}
}

/**
* Set user type.
*/
function vb_set_user_type($group_id)
{
	if ($group_id == 6)
	{
		return USER_FOUNDER;
	}
	else if ($group_id == 3)
	{
		// Group 3 is the Users Awaiting Email Confirmation group. These users not activated in phpBB.
		return USER_INACTIVE;
	}
	else
	{
		return USER_NORMAL;
	}
}

function vb_avatar($gallery_avatar_name)
{
	global $config, $convert_row;

	// If the filename field isn't empty, then we're using a custom avatar
	if (!empty($convert_row['filename']))
	{
		$user_id = phpbb_user_id($convert_row['userid']);
		$source = $convert_row['filename'];
		$ext = '.' . vb_file_ext($source);
		$user_avatar = $user_id . '_' . time() . $ext;

		if (VB_AVATAR_IN_DB)
		{
			$target =  $config['avatar_salt'] . '_' . $user_id . $ext;
			vb_write_file('avatar_path', $convert_row['filedata_thumb'], $target);
		}
		else
		{
			import_avatar($source, false, $user_id);
		}

		return $user_avatar;
	}
	else if (!empty($convert_row['avatarid']))
	{
		return $convert_row['avatarid'] . '.' . vb_file_ext($gallery_avatar_name);
	}

	return '';
}

function vb_avatar_type()
{
	global $convert_row;

	if (!empty($convert_row['filename']))
	{
		return AVATAR_UPLOAD;
	}
	else if (!empty($convert_row['avatarid']))
	{
		return AVATAR_GALLERY;
	}
	return 0;
}

/**
* Find out about the avatar's dimensions
*/
function vb_get_avatar_height($user_avatar)
{
	global $convert_row;

	if (!empty($convert_row['height_thumb']))
	{
		return (int) $convert_row['height_thumb'];
	}
	return get_avatar_height(basename($user_avatar), false, AVATAR_GALLERY);
}


/**
* Find out about the avatar's dimensions
*/
function vb_get_avatar_width($user_avatar)
{
	global $convert_row;

	if (!empty($convert_row['width_thumb']))
	{
		return (int) $convert_row['width_thumb'];
	}
	return get_avatar_width(basename($user_avatar), false, AVATAR_GALLERY);
}

/**
* Convert some user options
*/
function vb_user_options($options)
{
	global $convert_row;

	$convert_row = array_merge($convert_row, array(
		'popuppm'	=> ($options & 524288) ? 1 : 0,
		'viewsigs'	=> ($options & 1) ? 1 : 0,
		'viewimg'	=> ($options & 4) ? 1 : 0,
	));
}

/**
* Calculate the date a user became inactive
*/
function vb_inactive_time()
{
	global $convert_row;

	if ($convert_row['usergroupid'] == 3)
	{
		return $convert_row['joindate'];
	}
	return 0;
}

/**
* Calculate the reason a user became inactive
*/
function vb_inactive_reason()
{
	global $convert_row;

	if ($convert_row['usergroupid'] == 3)
	{
		return INACTIVE_REGISTER;
	}
	return 0;
}

/**
* Determine if the user wants to be autosubscribed.
*
* -1 = Not subscribed
* 0 = Subscribed, no email
* 1 = Instant email
* 2 = Daily email
* 3 = Weekly email
* 4 = Instant icq notification
*/
function vb_autosubscribe($type)
{
	return ($type > 0) ? 1 : 0;
}

/**
* Convert authentication
* user, group and forum table has to be filled in order to work
*/
function phpbb_convert_authentication($mode)
{
	global $db, $src_db, $same_db, $convert, $user, $config, $cache;

	if ($mode == 'start')
	{
		$db->sql_query($convert->truncate_statement . ACL_USERS_TABLE);
		$db->sql_query($convert->truncate_statement . ACL_GROUPS_TABLE);

		// What we will do is handling all 2.0.x admins as founder to replicate what is common in 2.0.x.
		// After conversion the main admin need to make sure he is removing permissions and the founder status if wanted.


		// Grab user ids of users with user_level of ADMIN
		$sql = "SELECT userid as user_id
			FROM {$convert->src_table_prefix}user
			WHERE usergroupid = 6
			ORDER BY joindate ASC";
		$result = $src_db->sql_query($sql);

		while ($row = $src_db->sql_fetchrow($result))
		{
			$user_id = (int) phpbb_user_id($row['user_id']);

			// Set founder admin...
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_type = ' . USER_FOUNDER . "
				WHERE user_id = $user_id";
			$db->sql_query($sql);
		}
		$src_db->sql_freeresult($result);
	}

	// Grab forum auth information
	$sql = "SELECT *
		FROM {$convert->src_table_prefix}forum";
	$result = $src_db->sql_query($sql);

	$forum_access = array();
	while ($row = $src_db->sql_fetchrow($result))
	{
		$forum_access[] = $row;
	}
	$src_db->sql_freeresult($result);

	if ($convert->mysql_convert && $same_db)
	{
		$src_db->sql_query("SET NAMES 'binary'");
	}
	// vb3 中好像没有使用用户权限，只使用了组权限，这儿直接提取版主权限
	// Grab user auth information from vB3.x board
	$sql="SELECT * from {$convert->src_table_prefix}moderator WHERE forumid > 0";
	$result = $src_db->sql_query($sql);

	$user_access = array();
	while ($row = $src_db->sql_fetchrow($result))
	{
		$user_access[$row['forumid']][] = $row;
	}
	$src_db->sql_freeresult($result);

	// Grab group auth information
	$sql = "SELECT *	FROM {$convert->src_table_prefix}forumpermission where usergroupid>1";
	$result = $src_db->sql_query($sql);

	$group_access = array();
	while ($row = $src_db->sql_fetchrow($result))
	{
		$group_access[$row['forumid']][] = $row;
	}
	$src_db->sql_freeresult($result);

	if ($convert->mysql_convert && $same_db)
	{
		$src_db->sql_query("SET NAMES 'utf8'");
	}

	// Add Forum Access List
	$auth_map = array(
		'auth_view'			=> array('f_', 'f_list'),
		'auth_read'			=> array('f_read', 'f_search'),
		'auth_post'			=> array('f_post', 'f_bbcode', 'f_smilies', 'f_img', 'f_sigs', 'f_postcount', 'f_report', 'f_subscribe', 'f_print', 'f_email'),
		'auth_reply'		=> 'f_reply',
		'auth_edit'			=> 'f_edit',
		'auth_delete'		=> 'f_delete',
		'auth_pollcreate'	=> 'f_poll',
		'auth_vote'			=> 'f_vote',
		'auth_announce'		=> 'f_announce',
		'auth_sticky'		=> 'f_sticky',
		'auth_attachments'	=> array('f_attach', 'f_download'),
		'auth_download'		=> 'f_download',
	);

	// Define the ACL constants used in 2.0 to make the code slightly more readable
	define('AUTH_ALL', 0);
	define('AUTH_REG', 1);
	define('AUTH_ACL', 2);
	define('AUTH_MOD', 3);
	define('AUTH_ADMIN', 5);

	// A mapping of the simple permissions used by 2.0
	$simple_auth_ary = array(
		'public'			=> array(
			'auth_view'			=> AUTH_ALL,
			'auth_read'			=> AUTH_ALL,
			'auth_post'			=> AUTH_ALL,
			'auth_reply'		=> AUTH_ALL,
			'auth_edit'			=> AUTH_REG,
			'auth_delete'		=> AUTH_REG,
			'auth_sticky'		=> AUTH_MOD,
			'auth_announce'		=> AUTH_MOD,
			'auth_vote'			=> AUTH_REG,
			'auth_pollcreate'	=> AUTH_REG,
		),
		'registered'		=> array(
			'auth_view'			=> AUTH_ALL,
			'auth_read'			=> AUTH_ALL,
			'auth_post'			=> AUTH_REG,
			'auth_reply'		=> AUTH_REG,
			'auth_edit'			=> AUTH_REG,
			'auth_delete'		=> AUTH_REG,
			'auth_sticky'		=> AUTH_MOD,
			'auth_announce'		=> AUTH_MOD,
			'auth_vote'			=> AUTH_REG,
			'auth_pollcreate'	=> AUTH_REG,
		),
		'registered_hidden'	=> array(
			'auth_view'			=> AUTH_REG,
			'auth_read'			=> AUTH_REG,
			'auth_post'			=> AUTH_REG,
			'auth_reply'		=> AUTH_REG,
			'auth_edit'			=> AUTH_REG,
			'auth_delete'		=> AUTH_REG,
			'auth_sticky'		=> AUTH_MOD,
			'auth_announce'		=> AUTH_MOD,
			'auth_vote'			=> AUTH_REG,
			'auth_pollcreate'	=> AUTH_REG,
		),
		'private'			=> array(
			'auth_view'			=> AUTH_ALL,
			'auth_read'			=> AUTH_ACL,
			'auth_post'			=> AUTH_ACL,
			'auth_reply'		=> AUTH_ACL,
			'auth_edit'			=> AUTH_ACL,
			'auth_delete'		=> AUTH_ACL,
			'auth_sticky'		=> AUTH_ACL,
			'auth_announce'		=> AUTH_MOD,
			'auth_vote'			=> AUTH_ACL,
			'auth_pollcreate'	=> AUTH_ACL,
		),
		'private_hidden'	=> array(
			'auth_view'			=> AUTH_ACL,
			'auth_read'			=> AUTH_ACL,
			'auth_post'			=> AUTH_ACL,
			'auth_reply'		=> AUTH_ACL,
			'auth_edit'			=> AUTH_ACL,
			'auth_delete'		=> AUTH_ACL,
			'auth_sticky'		=> AUTH_ACL,
			'auth_announce'		=> AUTH_MOD,
			'auth_vote'			=> AUTH_ACL,
			'auth_pollcreate'	=> AUTH_ACL,
		),
		'moderator'			=> array(
			'auth_view'			=> AUTH_ALL,
			'auth_read'			=> AUTH_MOD,
			'auth_post'			=> AUTH_MOD,
			'auth_reply'		=> AUTH_MOD,
			'auth_edit'			=> AUTH_MOD,
			'auth_delete'		=> AUTH_MOD,
			'auth_sticky'		=> AUTH_MOD,
			'auth_announce'		=> AUTH_MOD,
			'auth_vote'			=> AUTH_MOD,
			'auth_pollcreate'	=> AUTH_MOD,
		),
		'moderator_hidden'	=> array(
			'auth_view'			=> AUTH_MOD,
			'auth_read'			=> AUTH_MOD,
			'auth_post'			=> AUTH_MOD,
			'auth_reply'		=> AUTH_MOD,
			'auth_edit'			=> AUTH_MOD,
			'auth_delete'		=> AUTH_MOD,
			'auth_sticky'		=> AUTH_MOD,
			'auth_announce'		=> AUTH_MOD,
			'auth_vote'			=> AUTH_MOD,
			'auth_pollcreate'	=> AUTH_MOD,
		),
	);

	if ($mode == 'start')
	{
		user_group_auth('guests', 'SELECT user_id, {GUESTS} FROM ' . USERS_TABLE . ' WHERE user_id = ' . ANONYMOUS, false);
		user_group_auth('registered', 'SELECT user_id, {REGISTERED} FROM ' . USERS_TABLE . ' WHERE user_id <> ' . ANONYMOUS, false);

		// Selecting from old table
		if (!empty($config['increment_user_id']))
		{
			$auth_sql = 'SELECT userid as user_id, {ADMINISTRATORS} FROM ' . $convert->src_table_prefix . 'user WHERE usergroupid = 6 AND userid <> 1';
			user_group_auth('administrators', $auth_sql, true);

			$auth_sql = 'SELECT ' . $config['increment_user_id'] . ' as user_id, {ADMINISTRATORS} FROM ' . $convert->src_table_prefix . 'user WHERE usergroupid = 6 AND userid = 1';
			user_group_auth('administrators', $auth_sql, true);
		}
		else
		{
			$auth_sql = 'SELECT userid as user_id, {ADMINISTRATORS} FROM ' . $convert->src_table_prefix . 'user WHERE usergroupid = 6';
			user_group_auth('administrators', $auth_sql, true);
		}

		if (!empty($config['increment_user_id']))
		{
			$auth_sql = 'SELECT userid as user_id, {GLOBAL_MODERATORS} FROM ' . $convert->src_table_prefix . 'user WHERE usergroupid in (5,6) AND userid <> 1';
			user_group_auth('global_moderators', $auth_sql, true);

			$auth_sql = 'SELECT ' . $config['increment_user_id'] . ' as user_id, {GLOBAL_MODERATORS} FROM ' . $convert->src_table_prefix . 'user WHERE usergroupid in (5,6) AND userid = 1';
			user_group_auth('global_moderators', $auth_sql, true);
		}
		else
		{
			$auth_sql = 'SELECT userid as user_id, {GLOBAL_MODERATORS} FROM ' . $convert->src_table_prefix . 'user WHERE usergroupid in (5,6)';
			user_group_auth('global_moderators', $auth_sql, true);
		}
	}
	else if ($mode == 'first')
	{
		// Go through all vB3.x forums
		foreach ($forum_access as $forum)
		{
			$new_forum_id = (int) $forum['forumid'];

			// Administrators have full access to all forums whatever happens
			mass_auth('group_role', $new_forum_id, 'administrators', 'FORUM_FULL');
			//在这儿简单化处理，转换完毕后需到后台重新进行调整
			$matched_type = 'public';
			switch ($matched_type)
			{
				case 'public':
					mass_auth('group_role', $new_forum_id, 'guests', 'FORUM_LIMITED');
					mass_auth('group_role', $new_forum_id, 'registered', 'FORUM_LIMITED_POLLS');
					mass_auth('group_role', $new_forum_id, 'bots', 'FORUM_BOT');
				break;

				case 'registered':
					mass_auth('group_role', $new_forum_id, 'guests', 'FORUM_READONLY');
					mass_auth('group_role', $new_forum_id, 'bots', 'FORUM_BOT');

				// no break;

				case 'registered_hidden':
					mass_auth('group_role', $new_forum_id, 'registered', 'FORUM_POLLS');
				break;

				case 'private':
				case 'private_hidden':
				case 'moderator':
				case 'moderator_hidden':
				default:
					// The permissions don't match a simple set, so we're going to have to map them directly

					// No post approval for all, in 2.0.x this feature does not exist
					mass_auth('group', $new_forum_id, 'guests', 'f_noapprove', ACL_YES);
					mass_auth('group', $new_forum_id, 'registered', 'f_noapprove', ACL_YES);
				break;
			}
		}
	}
	else if ($mode == 'second')
	{
		// Assign permission roles and other default permissions

		// guests having u_download and u_search ability
		$db->sql_query('INSERT INTO ' . ACL_GROUPS_TABLE . ' (group_id, forum_id, auth_option_id, auth_role_id, auth_setting) SELECT ' . get_group_id('guests') . ', 0, auth_option_id, 0, 1 FROM ' . ACL_OPTIONS_TABLE . " WHERE auth_option IN ('u_', 'u_download', 'u_search')");

		// administrators/global mods having full user features
		mass_auth('group_role', 0, 'administrators', 'USER_FULL');
		mass_auth('group_role', 0, 'global_moderators', 'USER_FULL');

		// By default all converted administrators are given full access
		mass_auth('group_role', 0, 'administrators', 'ADMIN_FULL');

		// All registered users are assigned the standard user role
		mass_auth('group_role', 0, 'registered', 'USER_STANDARD');
		mass_auth('group_role', 0, 'registered_coppa', 'USER_STANDARD');

		// Instead of administrators being global moderators we give the MOD_FULL role to global mods (admins already assigned to this group)
		mass_auth('group_role', 0, 'global_moderators', 'MOD_FULL');
	}
	else if ($mode == 'third')
	{
		// And now the moderators
		// We make sure that they have at least standard access to the forums they moderate in addition to the moderating permissions
		foreach ($user_access as $forum_id => $access_map)
		{
			$forum_id = (int) $forum_id;

			foreach ($access_map as $access)
			{
				mass_auth('user_role', $forum_id, (int) phpbb_user_id($access['userid']), 'MOD_STANDARD');
				mass_auth('user_role', $forum_id, (int) phpbb_user_id($access['userid']), 'FORUM_STANDARD');
			}
		}

		foreach ($group_access as $forum_id => $access_map)
		{
			$forum_id = (int) $forum_id;

			foreach ($access_map as $access)
			{
				if (isset($access['forumpermissions']) && ($access['forumpermissions'] == 131072))
				{
					mass_auth('group_role', $forum_id, (int) $access['usergroupid'], 'MOD_STANDARD');
					mass_auth('group_role', $forum_id, (int) $access['usergroupid'], 'FORUM_STANDARD');
				}
			}
		}

		// We grant everyone readonly access to the categories to ensure that the forums are visible
		$sql = 'SELECT forum_id, forum_name, parent_id, left_id, right_id
			FROM ' . FORUMS_TABLE . '
			ORDER BY left_id ASC';
		$result = $db->sql_query($sql);

		$parent_forums = $forums = array();
		while ($row = $db->sql_fetchrow($result))
		{
			if ($row['parent_id'] == 0)
			{
				mass_auth('group_role', $row['forum_id'], 'administrators', 'FORUM_FULL');
				mass_auth('group_role', $row['forum_id'], 'global_moderators', 'FORUM_FULL');
				$parent_forums[] = $row;
			}
			else
			{
				$forums[] = $row;
			}
		}
		$db->sql_freeresult($result);

		global $auth;

		// Let us see which groups have access to these forums...
		foreach ($parent_forums as $row)
		{
			// Get the children
			$branch = $forum_ids = array();

			foreach ($forums as $key => $_row)
			{
				if ($_row['left_id'] > $row['left_id'] && $_row['left_id'] < $row['right_id'])
				{
					$branch[] = $_row;
					$forum_ids[] = $_row['forum_id'];
					continue;
				}
			}

			if (sizeof($forum_ids))
			{
				// Now make sure the user is able to read these forums
				$hold_ary = $auth->acl_group_raw_data(false, 'f_list', $forum_ids);

				if (empty($hold_ary))
				{
					continue;
				}

				foreach ($hold_ary as $g_id => $f_id_ary)
				{
					$set_group = false;

					foreach ($f_id_ary as $f_id => $auth_ary)
					{
						foreach ($auth_ary as $auth_option => $setting)
						{
							if ($setting == ACL_YES)
							{
								$set_group = true;
								break 2;
							}
						}
					}

					if ($set_group)
					{
						mass_auth('group', $row['forum_id'], $g_id, 'f_list', ACL_YES);
					}
				}
			}
		}
	}
}

/**
* Obtain list of forums in which different attachment categories can be used
*/
function phpbb_attachment_forum_perms($forum_permissions)
{
	if (empty($forum_permissions))
	{
		return '';
	}

	// Decode forum permissions
	$forum_ids = array();

	$one_char_encoding = '#';
	$two_char_encoding = '.';

	$auth_len = 1;
	for ($pos = 0; $pos < strlen($forum_permissions); $pos += $auth_len)
	{
		$forum_auth = substr($forum_permissions, $pos, 1);
		if ($forum_auth == $one_char_encoding)
		{
			$auth_len = 1;
			continue;
		}
		else if ($forum_auth == $two_char_encoding)
		{
			$auth_len = 2;
			$pos--;
			continue;
		}
		
		$forum_auth = substr($forum_permissions, $pos, $auth_len);
		$forum_id = base64_unpack($forum_auth);

		$forum_ids[] = (int) $forum_id;
	}
	
	if (sizeof($forum_ids))
	{
		return attachment_forum_perms($forum_ids);
	}

	return '';
}

/**
* Convert list of disallowed usernames
*/
function vb_convert_disallowed_usernames()
{
	global $convert, $db;

	$disallowed = trim(get_config_value('illegalusernames'));
	$disallowed = explode(' ', $disallowed);
	$insert_ary = array();

	foreach ($disallowed as $username)
	{
		$username = utf8_htmlspecialchars(vb_set_encoding($username));
		$username = '%' . $username . '%';
		$insert_ary[] = array('disallow_username' => $username);
	}

	$db->sql_query($convert->truncate_statement . DISALLOW_TABLE);

	if (sizeof($insert_ary))
	{
		$db->sql_multi_insert(DISALLOW_TABLE, $insert_ary);
	}
}

/**
* Checks whether there are any usernames on the old board that would map to the same
* username_clean on phpBB3. Prints out a list if any exist and exits.
*/
function phpbb_check_username_collisions()
{
	global $db, $src_db, $convert, $table_prefix, $user, $lang;

	$map_dbms = '';
	switch ($db->sql_layer)
	{
		case 'mysql':
			$map_dbms = 'mysql_40';
		break;
	
		case 'mysql4':
			if (version_compare($db->sql_server_info(true), '4.1.3', '>='))
			{
				$map_dbms = 'mysql_41';
			}
			else
			{
				$map_dbms = 'mysql_40';
			}
		break;
	
		case 'mysqli':
			$map_dbms = 'mysql_41';
		break;
	
		case 'mssql':
		case 'mssql_odbc':
			$map_dbms = 'mssql';
		break;
	
		default:
			$map_dbms = $db->sql_layer;
		break;
	}

	// create a temporary table in which we store the clean usernames
	$drop_sql = 'DROP TABLE ' . $table_prefix . 'userconv';
	switch ($map_dbms)
	{
		case 'firebird':
			$create_sql = 'CREATE TABLE ' . $table_prefix . 'userconv (
				user_id INTEGER NOT NULL,
				username_clean VARCHAR(255) CHARACTER SET UTF8 DEFAULT \'\' NOT NULL COLLATE UNICODE
			)';
		break;

		case 'mssql':
			$create_sql = 'CREATE TABLE [' . $table_prefix . 'userconv] (
				[user_id] [int] NOT NULL ,
				[username_clean] [varchar] (255) DEFAULT (\'\') NOT NULL
			)';
		break;

		case 'mysql_40':
			$create_sql = 'CREATE TABLE ' . $table_prefix . 'userconv (
				user_id mediumint(8) NOT NULL,
				username_clean blob NOT NULL
			)';
		break;

		case 'mysql_41':
			$create_sql = 'CREATE TABLE ' . $table_prefix . 'userconv (
				user_id mediumint(8) NOT NULL,
				username_clean varchar(255) DEFAULT \'\' NOT NULL
			) CHARACTER SET `utf8` COLLATE `utf8_bin`';
		break;

		case 'oracle':
			$create_sql = 'CREATE TABLE ' . $table_prefix . 'userconv
				user_id number(8) NOT NULL,
				username_clean varchar2(255) DEFAULT \'\'
			)';
		break;

		case 'postgres':
			$create_sql = 'CREATE TABLE ' . $table_prefix . 'userconv (
				user_id INT4 DEFAULT \'0\',
				username_clean varchar_ci DEFAULT \'\' NOT NULL
			)';
		break;

		case 'sqlite':
			$create_sql = 'CREATE TABLE ' . $table_prefix . 'userconv (
				user_id INTEGER NOT NULL DEFAULT \'0\',
				username_clean varchar(255) NOT NULL DEFAULT \'\',
			)';
		break;
	}

	$db->sql_return_on_error(true);
	$db->sql_query($drop_sql);
	$db->sql_query($create_sql);
	$db->sql_return_on_error(false);

	// now select all user_ids and usernames and then convert the username (this can take quite a while!)
	$sql = 'SELECT userid, username
		FROM ' . $convert->src_table_prefix . 'user';
	$result = $src_db->sql_query($sql);

	$insert_ary = array();
	$i = 0;
	while ($row = $src_db->sql_fetchrow($result))
	{
		$clean_name = utf8_clean_string(vb_set_encoding($row['username']));
		$insert_ary[] = array('user_id' => $row['userid'], 'username_clean' => $clean_name);

		if ($i % 1000 == 999)
		{
			$db->sql_multi_insert($table_prefix . 'userconv', $insert_ary);
			$insert_ary = array();
		}
		$i++;
	}
	$src_db->sql_freeresult($result);

	if (sizeof($insert_ary))
	{
		$db->sql_multi_insert($table_prefix . 'userconv', $insert_ary);
	}
	unset($insert_ary);

	// now find the clean version of the usernames that collide
	$sql = 'SELECT username_clean
		FROM ' . $table_prefix . 'userconv
		GROUP BY username_clean
		HAVING COUNT(user_id) > 1';
	$result = $db->sql_query($sql);

	$colliding_names = array();
	while ($row = $db->sql_fetchrow($result))
	{
		$colliding_names[] = $row['username_clean'];
	}
	$db->sql_freeresult($result);

	// there was at least one collision, the admin will have to solve it before conversion can continue
	if (sizeof($colliding_names))
	{
		$sql = 'SELECT user_id, username_clean
			FROM ' . $table_prefix . 'userconv
			WHERE ' . $db->sql_in_set('username_clean', $colliding_names);
		$result = $db->sql_query($sql);
		unset($colliding_names);

		$colliding_user_ids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$colliding_user_ids[(int) $row['user_id']] = $row['username_clean'];
		}
		$db->sql_freeresult($result);

		$sql = 'SELECT username, userid, posts
			FROM ' . $convert->src_table_prefix . 'user
			WHERE ' . $src_db->sql_in_set('userid', array_keys($colliding_user_ids));
		$result = $src_db->sql_query($sql);

		$colliding_users = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$row['userid'] = (int) $row['userid'];
			if (isset($colliding_user_ids[$row['userid']]))
			{
				$colliding_users[$colliding_user_ids[$row['userid']]][] = $row;
			}
		}
		$db->sql_freeresult($result);
		unset($colliding_user_ids);

		$list = '';
		foreach ($colliding_users as $username_clean => $users)
		{
			$list .= sprintf($user->lang['COLLIDING_CLEAN_USERNAME'], $username_clean) . "<br />\n";
			foreach ($users as $i => $row)
			{
				$list .= sprintf($user->lang['COLLIDING_USER'], $row['userid'], vb_set_encoding($row['username']), $row['posts']) . "<br />\n";
			}
		}

		$lang['INST_ERR_FATAL'] = $user->lang['CONV_ERR_FATAL'];
		$convert->p_master->error('<span style="color:red">' . $user->lang['COLLIDING_USERNAMES_FOUND'] . '</span></b><br /><br />' . $list . '<b>', __LINE__, __FILE__);
	}

	$db->sql_query($drop_sql);
}

/**
* Add password salt field to the users table.
*/
function vb_add_user_salt_field()
{
	global $db, $phpbb_root_path, $phpEx;

	if (!class_exists('phpbb_db_tools'))
	{
		include($phpbb_root_path . 'includes/db/db_tools.' . $phpEx);
	}

	$schema_changes = array(
		'add_columns'	=> array(
			USERS_TABLE	 => array(
				'user_passwd_salt'	=> array('VCHAR:30', ''),
			)
		)
	);

	$db_tools = new phpbb_db_tools($db);
	$db_tools->perform_schema_changes($schema_changes);
}

function add_bbcodes()
{
	global $db, $cache, $convert, $user;

	$add_bbcode_ary = array(
					'font=' => array(
						'bbcode_tag' 				=> 'font=',
    					'bbcode_match' 				=> '[font={SIMPLETEXT}]{TEXT}[/font]',
					    'bbcode_tpl' 				=> '{TEXT}',
    					'display_on_posting' 		=> '1',
					    'bbcode_helpline' 			=> '[font=Georgia]Georgia font[/font]',
					    'first_pass_match' 			=> '!\[font\=([a-zA-Z0-9-+.,_ ]+)\](.*?)\[/font\]!ies',
					    'first_pass_replace' 		=> '\'[font=${1}:$uid]\'.str_replace(array("\\r\\n", \'\\"\', \'\\\'\', \'(\', \')\'), array("\\n", \'"\', \'&#39;\', \'&#40;\', \'&#41;\'), trim(\'${2}\')).\'[/font:$uid]\'',
					    'second_pass_match' 		=> '!\[font\=([a-zA-Z0-9-+.,_ ]+):$uid\](.*?)\[/font:$uid\]!s',
					    'second_pass_replace' 		=> '<span style="font-family: ${1};">${2}</span>'
					),
					'align=' => array(
						'bbcode_tag' 				=> 'align=',
						'bbcode_match'				=> '[align={TEXT1}]{TEXT2}[/align]',
						'bbcode_tpl'				=> '<div style="text-align: {TEXT1};">{TEXT2}</div>',
						'display_on_posting'		=> '1',
						'bbcode_helpline'			=> 'Alignment: can use center, left, right',
						'first_pass_match'			=> '!\[align\=(.*?)\](.*?)\[/align\]!ies',
						'first_pass_replace'		=> '\'[align=${1}:$uid]\'.str_replace(array("\\r\\n", \'\\"\', \'\\\'\', \'(\', \')\'), array("\\n", \'"\', \'&#39;\', \'&#40;\', \'&#41;\'), trim(\'${2}\')).\'[/align:$uid]\'',
						'second_pass_match'			=> '!\[align\=(.*?):$uid\](.*?)\[/align:$uid\]!s',
						'second_pass_replace'		=> '<div style="text-align: ${1};">${2}</div>'
					)
			);

	foreach ($add_bbcode_ary as $v1) 
	{
		$sql_ary =array();

		foreach ($v1 as $k => $v) 
		{
			$sql_ary[$k] = $v;
		}
		$sql = 'SELECT 1 as test
			FROM ' . BBCODES_TABLE . "
			WHERE LOWER(bbcode_tag) = '" . $db->sql_escape(strtolower($sql_ary['bbcode_tag'])) . "'";
		$result = $db->sql_query($sql);
		$info = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		// If the bbcode does not already exist?
		if ($info['test'] != '1')
		{
			$sql = 'SELECT MAX(bbcode_id) as max_bbcode_id
				FROM ' . BBCODES_TABLE;
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if ($row)
			{
				$bbcode_id = $row['max_bbcode_id'] + 1;

				// Make sure it is greater than the core bbcode ids...
				if ($bbcode_id <= NUM_CORE_BBCODES)
				{
					$bbcode_id = NUM_CORE_BBCODES + 1;
				}
			}
			else
			{
				$bbcode_id = NUM_CORE_BBCODES + 1;
			}
			// 1511 is the maximum bbcode_id allowed.
			if ($bbcode_id < 1512)
			{
				$sql_ary['bbcode_id'] = (int) $bbcode_id;

				$db->sql_query('INSERT INTO ' . BBCODES_TABLE . $db->sql_build_array('INSERT', $sql_ary));
				$cache->destroy('sql', BBCODES_TABLE);
			}
		}
	}
}
?>
